@@ -143,6 +143,9 @@ struct _BonoboMDIPrivate
 
 	gint 		 default_window_height;
 	gint		 default_window_width;
+
+	/* Whether our state is being restored for session management */
+	gboolean	 restoring_state;
 };
 
 typedef gboolean   (*BonoboMDISignal1) (GObject *, gpointer, gpointer);
@@ -441,7 +444,14 @@ child_list_activated_cb (BonoboUIComponent *uic, gpointer user_data, const gchar
 		GList *views = bonobo_mdi_child_get_views (child);
 		
 		if (views)
+		{
+			GtkWindow *window;
+
+			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (views->data));
+			gtk_window_present (window);
+
 			bonobo_mdi_set_active_view (mdi, views->data);
+		}
 		else
 			bonobo_mdi_add_view (mdi, child);
 	}
@@ -510,11 +520,16 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 		gchar *verb_name = NULL;
 		gchar *tip;
 		gchar *escaped_name;
+		gchar *safe_name;
 		gchar *child_name = bonobo_mdi_child_get_name (BONOBO_MDI_CHILD (child->data));
 
-		escaped_name = escape_underscores (child_name);
-		
-		tip =  g_strdup_printf (_("Activate %s"), child_name);
+		safe_name = g_markup_escape_text (child_name, strlen (child_name));
+		g_return_if_fail (safe_name != NULL);
+
+		escaped_name = escape_underscores (safe_name);
+		g_return_if_fail (escaped_name != NULL);
+
+		tip =  g_strdup_printf (_("Activate %s"), safe_name);
 		verb_name = g_strdup_printf ("Child_%p", child->data);
 		xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
 				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
@@ -523,6 +538,7 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 
 		g_free (tip);
 		g_free (child_name);
+		g_free (safe_name);
 		g_free (escaped_name);
 
 		bonobo_ui_component_set_translate (ui_component, mdi->priv->child_list_path, xml, NULL);
@@ -582,7 +598,7 @@ void
 child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 {
 	GList *win_node;
-	gchar *child_name, *escaped_name;
+	gchar *child_name, *escaped_name, *safe_name;
 	gchar* xml, *cmd, *verb_name, *tip;
 	int accel_num;
 	
@@ -594,10 +610,16 @@ child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 	win_node = mdi->priv->windows;
 
 	child_name = bonobo_mdi_child_get_name (child);
-	escaped_name = escape_underscores (child_name);
+
+	safe_name = g_markup_escape_text (child_name, strlen (child_name));
+	g_return_if_fail (safe_name != NULL);
+
+	escaped_name = escape_underscores (safe_name);
+	g_return_if_fail (escaped_name != NULL);
+
 	verb_name = g_strdup_printf ("Child_%p", child);
 	
-	tip = g_strdup_printf (_("Activate %s"), child_name);
+	tip = g_strdup_printf (_("Activate %s"), safe_name);
 	xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
 				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
 
@@ -631,6 +653,7 @@ child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 	g_free (xml);
 	g_free (cmd);
 	g_free (verb_name);
+	g_free (safe_name);
 }
 
 static gboolean 
@@ -972,7 +995,7 @@ app_clone (BonoboMDI *mdi, BonoboWindow *win, const char *window_role)
 	
 	app_create (mdi, NULL, window_role);
 
-	if (win != NULL)
+	if (win != NULL && !bonobo_mdi_get_restoring_state (mdi))
 	{
 		const BonoboMDIWindowInfo *window_info = bonobo_mdi_get_window_info (win);
 		g_return_if_fail (window_info != NULL);
@@ -1176,9 +1199,10 @@ app_create (BonoboMDI *mdi, gchar *layout_string, const char *window_role)
 	window = bonobo_window_new (mdi->priv->mdi_name, mdi->priv->title);
 	g_return_if_fail (window != NULL);
 
-	gtk_window_set_default_size (GTK_WINDOW (window),
-				     mdi->priv->default_window_width,
-				     mdi->priv->default_window_height);
+	if (!bonobo_mdi_get_restoring_state (mdi))
+		gtk_window_set_default_size (GTK_WINDOW (window),
+					     mdi->priv->default_window_width,
+					     mdi->priv->default_window_height);
 
 	if (window_role)
 		gtk_window_set_role (GTK_WINDOW (window), window_role);
@@ -1305,8 +1329,7 @@ set_active_view (BonoboMDI *mdi, GtkWidget *view)
  * @view: A pointer to the view that is to become the active one.
  * 
  * Description:
- * Sets the active view to @view. It also raises the window containing it
- * and gives it focus.
+ * Sets the active view to @view.
  **/
 void 
 bonobo_mdi_set_active_view (BonoboMDI *mdi, GtkWidget *view)
@@ -1324,8 +1347,6 @@ bonobo_mdi_set_active_view (BonoboMDI *mdi, GtkWidget *view)
 	
 	window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
 	
-	gtk_window_present (window);
-
 	set_active_view (mdi, view);
 
 	gedit_debug (DEBUG_MDI, "END");
@@ -2182,3 +2203,46 @@ bonobo_mdi_get_window_info (BonoboWindow *win)
 	return (const BonoboMDIWindowInfo *) 
 			g_object_get_data (G_OBJECT (win), WINDOW_INFO_KEY);
 }
+
+/**
+ * bonobo_mdi_set_restoring_state:
+ * @mdi: A pointer to an MDI object.
+ * @restoring_state: Whether the state is being restored.
+ * 
+ * Sets whether the MDI object is being restored to a known state.
+ **/
+void
+bonobo_mdi_set_restoring_state (BonoboMDI *mdi, gboolean restoring_state)
+{
+	BonoboMDIPrivate *priv;
+
+	g_return_if_fail (mdi != NULL);
+	g_return_if_fail (BONOBO_IS_MDI (mdi));
+
+	priv = mdi->priv;
+
+	g_return_if_fail (priv->restoring_state && restoring_state);
+	g_return_if_fail (!priv->restoring_state && !restoring_state);
+
+	priv->restoring_state = restoring_state ? TRUE : FALSE;
+}
+
+/**
+ * bonobo_mdi_get_restoring_state:
+ * @mdi: A pointer to an MDI object.
+ * 
+ * Queries whether an MDI object is having its state restored.
+ * 
+ * Return value: TRUE if the MDI object is being restored, FALSE otherwise.
+ **/
+gboolean
+bonobo_mdi_get_restoring_state (BonoboMDI *mdi)
+{
+	BonoboMDIPrivate *priv;
+
+	g_return_val_if_fail (mdi != NULL, FALSE);
+	g_return_val_if_fail (BONOBO_IS_MDI (mdi), FALSE);
+
+	priv = mdi->priv;
+	return priv->restoring_state;
+}
