@@ -51,6 +51,8 @@
 #include <bonobo/bonobo-file-selector-util.h>
 
 #include <string.h>
+#include <unistd.h>
+#include <errno.h>
 
 #include <bonobo/bonobo-event-source.h>
 #include <bonobo/bonobo-exception.h>
@@ -67,8 +69,9 @@
 #include <bonobo/bonobo-i18n.h>
 
 #include <libgnomevfs/gnome-vfs.h>
+#include <eel/eel-vfs-extensions.h>
+#include <eel/eel-string.h>
 
-#include "gnome-vfs-helpers.h"
 #include "gedit-utils.h"
 
 static GQuark user_data_id = 0;
@@ -81,23 +84,41 @@ delete_file_selector (GtkWidget *d, GdkEventAny *e, gpointer data)
 	return TRUE;
 }
 
+/* Displays a confirmation dialog for whether to replace a file.  The message
+ * should contain a %s to include the file name.
+ */
 static gboolean
-replace_existing_file (GtkWindow *parent, const gchar* file_name)
+replace_dialog (GtkWindow *parent, const gchar *message, const gchar *file_name)
 {
 	GtkWidget *msgbox;
 	gint ret;
-	gchar* uri;
-	
-	uri = gnome_vfs_x_format_uri_for_display (file_name);
-	msgbox = gtk_message_dialog_new (parent,
-			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_QUESTION,
-			GTK_BUTTONS_NONE,
-			_("A file named ''%s'' already exists.\n"
-			  "Do you want to replace it with the "
-			  "one you are saving?"), 
-			uri);
+	gchar *full_formatted_uri;
+       	gchar *uri_for_display	;
+	gchar *uri;
+
+	uri = eel_make_uri_from_shell_arg (file_name);
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	full_formatted_uri = eel_format_uri_for_display (uri);
+	g_return_val_if_fail (full_formatted_uri != NULL, FALSE);
 	g_free (uri);
+	
+	/* Truncate the URI so it doesn't get insanely wide. Note that even
+	 * though the dialog uses wrapped text, if the URI doesn't contain
+	 * white space then the text-wrapping code is too stupid to wrap it.
+	 */
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 50);
+	g_return_val_if_fail (uri_for_display != NULL, FALSE);
+
+	g_free (full_formatted_uri);
+
+	msgbox = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 message, 
+					 uri_for_display);
+	g_free (uri_for_display);
 
 	/* Add Don't Replace button */
 	gedit_dialog_add_button (GTK_DIALOG (msgbox), 
@@ -118,6 +139,28 @@ replace_existing_file (GtkWindow *parent, const gchar* file_name)
 	return (ret == GTK_RESPONSE_YES);
 }
 
+/* Presents a confirmation dialog for whether to replace an existing file */
+static gboolean
+replace_existing_file (GtkWindow *parent, const gchar* file_name)
+{
+	return replace_dialog (parent,
+			       _("A file named \"%s\" already exists.\n"
+				 "Do you want to replace it with the "
+				 "one you are saving?"),
+			       file_name);
+}
+
+/* Presents a confirmation dialog for whether to replace a read-only file */
+static gboolean
+replace_read_only_file (GtkWindow *parent, const gchar* file_name)
+{
+	return replace_dialog (parent,
+			       _("The file \"%s\" is read-only.\n"
+				 "Do you want to try to replace it with the "
+				 "one you are saving?"),
+			       file_name);
+}
+
 static void
 listener_cb (BonoboListener *listener, 
 	     const gchar *event_name,
@@ -250,6 +293,16 @@ concat_dir_and_file (const char *dir, const char *file)
 		return g_strconcat (dir, file, NULL);
 }
 
+/* Tests whether we have write permissions for a file */
+static gboolean
+is_read_only (const gchar *filename)
+{
+	if (access (filename, W_OK) == -1)
+		return (errno == EACCES);
+	else
+		return FALSE;
+}
+
 static void
 ok_clicked_cb (GtkWidget *widget, gpointer data)
 {
@@ -280,9 +333,16 @@ ok_clicked_cb (GtkWidget *widget, gpointer data)
 	} 
 	else
 	{	
-		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) 
-			if (!replace_existing_file (GTK_WINDOW (fsel), file_name))
+		if (g_file_test (file_name, G_FILE_TEST_EXISTS))
+		{
+			if (is_read_only (file_name))
+			{
+				if (!replace_read_only_file (GTK_WINDOW (fsel), file_name))
+					return;
+			}
+			else if (!replace_existing_file (GTK_WINDOW (fsel), file_name))
 				return;
+		}
 
 		gtk_widget_hide (GTK_WIDGET (fsel));
 
